@@ -1,10 +1,29 @@
-from flask import request
+from flask import jsonify, request, current_app
 from datetime import date
 from polzybackend.authenticate import bp
+from polzybackend.utils.import_utils import all_stages, permissions
 from polzybackend.models import User
 from polzybackend import auth
 
 
+@bp.route('/stages')
+def stages():
+    #
+    # returns list of all available stages for login
+    #
+
+    try:
+        # get all stages
+        stages = all_stages()()
+        current_app.logger.debug(f"Value of stages: {stages}")
+
+    except Exception as e:
+        current_app.logger.warning(f'Failed to get All Stages: {e}')
+        stages = []
+
+    return jsonify(stages), 200
+
+
 @bp.route('/login', methods=['POST'])
 def login():
     # get request data
@@ -32,16 +51,12 @@ def login():
     user.set_stage(data['stage'])
     user.set_language(data['language'])
 
-    # update company if user has only one
-    if len(user.companies) == 1:
-        user.set_company(company=user.companies[0].company)
-
-    return user.to_json(), 200
+    return jsonify(user.to_json()), 200
 
 
-@bp.route('/company', methods=['POST'])
+@bp.route('/permissions', methods=['POST'])
 @auth.login_required
-def set_current_company():
+def user_permissions():
     # get request data
     data = request.get_json()
 
@@ -51,11 +66,13 @@ def set_current_company():
         return {'error': 'Company data does not contain company id'}, 400
 
     # update company
+    user = auth.current_user()
     try:
-        auth.current_user().set_company(company_id=company_id)
+        company_details = user.set_company(company_id=company_id)
     except Exception as e:
         return {'error': f'Set company failed: {e}'}, 400
 
-    return {'success': 'Success'}, 200
-
-
+    return jsonify({
+        'permissions': permissions(user),
+        'company': company_details,
+    }), 200
